'use strict';

var cc = require('./core');


var top = function(deferred) {
  deferred.then(null, function(ex) { console.log(ex.stack); });
};


var join = function(items) {
  return cc.go(wrapGenerator.mark(function() {
    var result, i;

    return wrapGenerator(function($ctx0) {
      while (1) switch ($ctx0.next) {
      case 0:
        result = [];
        $ctx0.t0 = $ctx0.keys(items);
      case 2:
        if (!$ctx0.t0.length) {
          $ctx0.next = 10;
          break;
        }

        i = $ctx0.t0.pop();
        $ctx0.next = 6;
        return items[i];
      case 6:
        $ctx0.t1 = $ctx0.sent;
        result.push($ctx0.t1);
        $ctx0.next = 2;
        break;
      case 10:
        $ctx0.rval = result;
        delete $ctx0.thrown;
        $ctx0.next = 14;
        break;
      case 14:
      case "end":
        return $ctx0.stop();
      }
    }, this);
  }));
};


var lift = function(fn, context) {
  return function() {
    var args = Array.prototype.slice.call(arguments);
    return cc.go(wrapGenerator.mark(function() {
      return wrapGenerator(function($ctx1) {
        while (1) switch ($ctx1.next) {
        case 0:
          $ctx1.next = 2;
          return join(args);
        case 2:
          $ctx1.t2 = $ctx1.sent;
          $ctx1.rval = fn.apply(context, $ctx1.t2);
          delete $ctx1.thrown;
          $ctx1.next = 7;
          break;
        case 7:
        case "end":
          return $ctx1.stop();
        }
      }, this);
    }));
  };
};


var chain = function(initial) {
  var args = Array.prototype.slice.call(arguments, 1);

  return cc.go(wrapGenerator.mark(function() {
    var val, form, i;

    return wrapGenerator(function($ctx2) {
      while (1) switch ($ctx2.next) {
      case 0:
        val = initial;
        i = 0;
      case 2:
        if (!(i < args.length)) {
          $ctx2.next = 11;
          break;
        }

        form = args[i];
        $ctx2.next = 6;
        return val;
      case 6:
        val = $ctx2.sent;

        if (typeof form == 'function')
          val = form(val);
        else if (Array.isArray(form) && typeof form[0] == 'function')
          val = form[0].apply(null, [].concat(form[1], [val], form.slice(2)));
        else if (Array.isArray(form) && typeof form[0] == 'string')
          val = val[form[0]].apply(val, form.slice(1));
        else
          val = form;
      case 8:
        ++i;
        $ctx2.next = 2;
        break;
      case 11:
        $ctx2.next = 13;
        return val;
      case 13:
        $ctx2.rval = $ctx2.sent;
        delete $ctx2.thrown;
        $ctx2.next = 17;
        break;
      case 17:
      case "end":
        return $ctx2.stop();
      }
    }, this);
  }));
};


var sleep = function(ms) {
  var result = cc.defer();
  var t = setTimeout(function() {
    clearTimeout(t);
    result.resolve();
  }, ms);
  return result;
};


var ncallback = function(deferred) {
  return function(err, val) {
    if (err)
      deferred.reject(new Error(err));
    else
      deferred.resolve(val);
  };
};


var nbind = function(fn, context) {
  var boundArgs = Array.prototype.slice.call(arguments, 2);

  return function() {
    var args = Array.prototype.slice.call(arguments);
    var result = cc.defer();

    fn.apply(context, boundArgs.concat(args, ncallback(result)));

    return result;
  };
};


var nodeify = function(deferred, callback) {
  if (callback === undefined)
    return deferred;
  else {
    deferred.then(function(val) {
      callback(null, val)
    }, function(err) {
      callback(err);
    });
  }
};


module.exports = {
  top      : top,
  join     : join,
  lift     : lift,
  chain    : chain,
  sleep    : sleep,
  ncallback: ncallback,
  nbind    : nbind,
  nodeify  : nodeify
};
